==> abstract_shapes-0.0.0.32-py3-none-any.whl: abstract_shapes/direction.py <==
from shapely.geometry import Point, Polygon, MultiPolygon
# Importing RotatingFileHandler
def coord_extractor(coord,latitude=True):
    def get_coord(coord,latitude):
        return coord[1] if latitude else coord[0]
    if isinstance(coord,list):
        return [get_coord(cord,latitude) for cord in coord]
            
    return get_coord(coord,latitude)
def get_position_of_geom(geom, r=max, latitude=True):
    """
    Find the position (either min or max) of a geometry by latitude or longitude.

    Args:
        geom: A shapely geometry object (Polygon, MultiPolygon, or Point).
        r: A function to apply (min or max).
        latitude: If True, find the point by latitude; otherwise, by longitude.

    Returns:
        A tuple representing the coordinates of the desired position.
    """
    

    if isinstance(geom, Point):
        return (geom.x, geom.y)
    elif isinstance(geom, Polygon):
        coords = list(geom.exterior.coords)
    elif isinstance(geom, MultiPolygon):
        coords = [coord for polygon in geom.geoms for coord in polygon.exterior.coords]
    else:
        raise ValueError("Unsupported geometry type.")

    # Extract the desired coordinate based on latitude or longitude
    desired_coord = r(coords, key=coord_extractor)
    return desired_coord

def get_extremity(geom_a, geom_b, latitude=True):
    """
    Determine which of two geometries has the most extreme point (north/south or east/west).

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: If True, compares by latitude (north/south); otherwise, by longitude (east/west).

    Returns:
        A dictionary containing the geometries and their extreme points.
    """
    comparison_key = 1 if latitude else 0

    # Choose comparison function based on direction
    r = max if latitude else min

    max_a = get_position_of_geom(geom_a, r=r, latitude=latitude)
    max_b = get_position_of_geom(geom_b, r=r, latitude=latitude)

    is_a_more_extreme = max_a[comparison_key] > max_b[comparison_key] if latitude else max_a[comparison_key] < max_b[comparison_key]
    return {
        "max": {
            "geom": 'a' if is_a_more_extreme else 'b',
            "geometry": geom_a if is_a_more_extreme else geom_b,
            "coordinates": max_a if is_a_more_extreme else max_b,
            'direction': latitude
        },
        "min": {
            "geom": 'b' if is_a_more_extreme else 'a',
            "geometry": geom_b if is_a_more_extreme else geom_a,
            "coordinates": max_b if is_a_more_extreme else max_a,
            'direction': latitude
        }
    }

def get_auto_most(geom_a, geom_b, latitude='north'):
    """
    Determine the most extreme geometry based on cardinal direction.

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: One of 'north', 'south', 'east', 'west'.

    Returns:
        A dictionary containing the geometries and their extreme points.
    """
    latitude_lower = str(latitude).lower()
    if latitude_lower in ['south', 'west']:
        r = min
    else:
        r = max

    latitude = latitude_lower in ['north', 'south']
    return get_extremity(geom_a, geom_b, latitude=latitude)

def get_geom_range(geom_a, geom_b, latitude='north', rangeInclusiveA=False, rangeInclusiveB=False,strictA=False,strict_B=False):
    """
    Determine the range between two geometries based on the specified direction.

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: One of 'north', 'south', 'east', 'west'.
        rangeInclusiveA: Boolean specifying whether the first geometry should include its extremity.
        rangeInclusiveB: Boolean specifying whether the second geometry should include its extremity.

    Returns:
        A dictionary containing the directional range limits and their corresponding coordinates.
    """
    reference_js = {"a": {"geom": geom_a, "inclusive": rangeInclusiveA,"strict":strictA}, "b": {"geom": geom_b, "inclusive": rangeInclusiveB,"strict":strictB}}
    auto_js = get_auto_most(geom_a, geom_b, latitude=latitude)
    
    # Determine the reference functions (min or max) based on inclusivity
    r_a = max if bool(reference_js[auto_js['max']['geom']]["inclusive"]) else min
    r_b = min if bool(reference_js[auto_js['min']['geom']]["inclusive"]) else max

    return {
        "latitude": [auto_js['min']['direction'], auto_js['max']['direction']],
        "strict":[reference_js[auto_js['min']['geom']]['strict'],reference_js[auto_js['max']['geom']]['strict']],
        "range": [
            get_position_of_geom(reference_js[auto_js['min']['geom']]["geom"], r=r_b, latitude=auto_js['min']['direction']),
            get_position_of_geom(reference_js[auto_js['max']['geom']]["geom"], r=r_a, latitude=auto_js['max']['direction'])
        ]
    }

# Function to check if a geometry's latitude lies within specified bounds, supporting both Polygon and MultiPolygon
def is_within_bounds(geometry, geom_range):
    def extract_latitudes(polygon):
        """Extract latitudes from a polygon's exterior coordinates."""
        return [coord_extractor(coord,latitude=latitude) for coord in polygon.exterior.coords]
    
    lower_bound,upper_bound=coord_extractor(geom_range["range"][0],geom_range["latitude"][0])mcoord_extractor(geom_range["range"][1],geom_range["latitude"][1])
    geom_range["strict"][0] = bool(geom_range["strict"][0])
    geom_range["strict"][1]=bool(geom_range["strict"][1])
    
    latitudes = []
    if isinstance(geometry, Polygon):
        latitudes = extract_latitudes(geometry)
    elif isinstance(geometry, MultiPolygon):
        for polygon in geometry.geoms:
            latitudes.extend(extract_latitudes(polygon))
    else:
        raise TypeError("Unsupported geometry type")
    if geom_range["strict"][0] and geom_range["strict"][1]:
        if any((lower_bound > lat or upper_bound < lat) for lat in latitudes):
            return False   
    elif geom_range["strict"][0] and not geom_range["strict"][1]:
        if any(lower_bound > lat for lat in latitudes):
            return False
    elif geom_range["strict"][1] and not geom_range["strict"][0]:
        if any(upper_bound < lat for lat in latitudes):
            return False
    # Check if any latitude falls within the specified bounds
    return any(lower_bound <= lat <= upper_bound for lat in latitudes)

==> abstract_shapes-0.0.0.37-py3-none-any.whl: abstract_shapes/direction.py <==
from shapely.geometry import Point, Polygon, MultiPolygon
# Importing RotatingFileHandler
def coord_extractor(coord,latitude=True):
    def get_coord(coord,latitude):
        return coord[1] if latitude else coord[0]
    if isinstance(coord,list):
        return [get_coord(cord,latitude) for cord in coord]
            
    return get_coord(coord,latitude)
def get_position_of_geom(geom, r=max, latitude=True):
    """
    Find the position (either min or max) of a geometry by latitude or longitude.

    Args:
        geom: A shapely geometry object (Polygon, MultiPolygon, or Point).
        r: A function to apply (min or max).
        latitude: If True, find the point by latitude; otherwise, by longitude.

    Returns:
        A tuple representing the coordinates of the desired position.
    """
    

    if isinstance(geom, Point):
        return (geom.x, geom.y)
    elif isinstance(geom, Polygon):
        coords = list(geom.exterior.coords)
    elif isinstance(geom, MultiPolygon):
        coords = [coord for polygon in geom.geoms for coord in polygon.exterior.coords]
    else:
        raise ValueError("Unsupported geometry type.")

    # Extract the desired coordinate based on latitude or longitude
    desired_coord = r(coords, key=coord_extractor)
    return desired_coord

def get_extremity(geom_a, geom_b, latitude=True):
    """
    Determine which of two geometries has the most extreme point (north/south or east/west).

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: If True, compares by latitude (north/south); otherwise, by longitude (east/west).

    Returns:
        A dictionary containing the geometries and their extreme points.
    """
    comparison_key = 1 if latitude else 0

    # Choose comparison function based on direction
    r = max if latitude else min

    max_a = get_position_of_geom(geom_a, r=r, latitude=latitude)
    max_b = get_position_of_geom(geom_b, r=r, latitude=latitude)

    is_a_more_extreme = max_a[comparison_key] > max_b[comparison_key] if latitude else max_a[comparison_key] < max_b[comparison_key]
    return {
        "max": {
            "geom": 'a' if is_a_more_extreme else 'b',
            "geometry": geom_a if is_a_more_extreme else geom_b,
            "coordinates": max_a if is_a_more_extreme else max_b,
            'direction': latitude
        },
        "min": {
            "geom": 'b' if is_a_more_extreme else 'a',
            "geometry": geom_b if is_a_more_extreme else geom_a,
            "coordinates": max_b if is_a_more_extreme else max_a,
            'direction': latitude
        }
    }

def get_auto_most(geom_a, geom_b, latitude='north'):
    """
    Determine the most extreme geometry based on cardinal direction.

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: One of 'north', 'south', 'east', 'west'.

    Returns:
        A dictionary containing the geometries and their extreme points.
    """
    latitude_lower = str(latitude).lower()
    if latitude_lower in ['south', 'west']:
        r = min
    else:
        r = max

    latitude = latitude_lower in ['north', 'south']
    return get_extremity(geom_a, geom_b, latitude=latitude)

def get_geom_range(geom_a, geom_b, latitude='north', rangeInclusiveA=False, rangeInclusiveB=False,strictA=False,strictB=False):
    """
    Determine the range between two geometries based on the specified direction.

    Args:
        geom_a: A shapely geometry object.
        geom_b: Another shapely geometry object.
        latitude: One of 'north', 'south', 'east', 'west'.
        rangeInclusiveA: Boolean specifying whether the first geometry should include its extremity.
        rangeInclusiveB: Boolean specifying whether the second geometry should include its extremity.

    Returns:
        A dictionary containing the directional range limits and their corresponding coordinates.
    """
    reference_js = {"a": {"geom": geom_a, "inclusive": rangeInclusiveA,"strict":strictA}, "b": {"geom": geom_b, "inclusive": rangeInclusiveB,"strict":strictB}}
    auto_js = get_auto_most(geom_a, geom_b, latitude=latitude)
    
    # Determine the reference functions (min or max) based on inclusivity
    r_a = max if bool(reference_js[auto_js['max']['geom']]["inclusive"]) else min
    r_b = min if bool(reference_js[auto_js['min']['geom']]["inclusive"]) else max

    return {
        "latitude": [auto_js['min']['direction'], auto_js['max']['direction']],
        "strict":[bool(reference_js[auto_js['min']['geom']]['strict']),bool(reference_js[auto_js['max']['geom']]['strict'])],
        "range": [
            get_position_of_geom(reference_js[auto_js['min']['geom']]["geom"], r=r_b, latitude=auto_js['min']['direction']),
            get_position_of_geom(reference_js[auto_js['max']['geom']]["geom"], r=r_a, latitude=auto_js['max']['direction'])
        ]
    }

# Function to check if a geometry's latitude lies within specified bounds, supporting both Polygon and MultiPolygon
def is_geom_within_bounds(geometry, geom_range):
    def extract_latitudes(polygon):
        """Extract latitudes from a polygon's exterior coordinates."""
        return [coord_extractor(coord,latitude=latitude) for coord in polygon.exterior.coords]
    
    lower_bound,upper_bound=coord_extractor(geom_range["range"][0],geom_range["latitude"][0]),coord_extractor(geom_range["range"][1],geom_range["latitude"][1])
    latitudes = []
    for i,lattitude in enumerate(geom_range["latitude"]):
        if i==1 and lattitude==geom_range["latitude"][0]:

            latitudes.extend(latitudes[0])
            break
        if isinstance(geometry, Polygon):
            latitudes = extract_latitudes(geometry,lattitude)
        elif isinstance(geometry, MultiPolygon):
            for polygon in geometry.geoms:
            latitudes.extend(extract_latitudes(polygon,lattitude))
        else:
            raise TypeError("Unsupported geometry type")
    if geom_range["strict"][0] and geom_range["strict"][1]:
        if lattitudes[0]==lattitudes[1]:
            if any((lower_bound > lat or upper_bound < lat) for lat in latitudes):
                return False
        else:
            if any(lower_bound > lat for lat in lattitudes[0]) or any(higher_bound < lat for lat in lattitudes[1]):
                return False   
    elif geom_range["strict"][0] and not geom_range["strict"][1]:
        if any(lower_bound > lat for lat in lattitudes[0]):
            return False
    elif geom_range["strict"][1] and not geom_range["strict"][0]:
        if any(upper_bound < lat for lat in lattitudes[1]):
            return False
    # Check if any latitude falls within the specified bounds
    else:
         if lattitudes[0]==lattitudes[1]:
            return any(lower_bound <= lat <= upper_bound for lat in latitudes)
         else:
             return (any(ulower_bound <= lat for lat in lattitudes[0]) and any(lat  <= upper_bound for lat in lattitudes[1]))
    return False
